<?php
/*
Plugin Name: Customerlabs CDP
Description: A simple implementation of e-commerce events tracking for Wordpress
Version: 1.8.1
License: GPLv2
Author: CustomerLabs Digital Solutions Pvt. Ltd.
Author URI: https://customerlabs.co
Author Email: team@customerlabs.co
*/


class CLTracker {
    /**
    * The singleton instance of CLTracker.
	*
    * @access private
	* @var CLTracker
	*/
    
    private static $instance;

    /**
	* Retrieves the one true instance of CLTracker
	*
	* @since  beta
	* @return object Singleton instance of CLTracker
	*/
	public static function get_instance() {

		if ( ! isset( self::$instance ) ) {

			self::$instance = new CLTracker;

		}

		return self::$instance;
    }
    

    /**
	* Load Customerlabs tracking code.
	*
	*
	* @param  array $settings Settings options array.
	*/
	public static function initialize( $settings ) {

		if ( ! isset( $settings['app_id'] ) || $settings['app_id'] == '' ) {
			return;
		}

		include_once( dirname( __FILE__ ) . '/templates/tracking_code.php' );

	}
    

    /**
	* Load Javascript `identify` call
	*
    *
	* @param  int|string  $user_id currently we are using email as customer user id.
	* @param  array       $traits  Array of traits to pass to Customerlabs.
	*/
	public static function identify( $traits = array() ) {

		include_once( dirname( __FILE__ ) . '/templates/identify.php' );
	}


    /**
	* Load Javascript `track` call
	*
	*
	* @param  array  $tracks       An array of events to send to CustomerLabs.
	*
	*/
	public static function track( $tracks, $settings ) {

		include_once( dirname( __FILE__ ) . '/templates/track.php' );
	}
}

class CLTracker_WordPress {

    /**
	* Slug used in page and menu names
	*/
	const SLUG    = 'cltracker';

	/**
	* Current plugin version.
	*/
	const VERSION = 'beta';

	/**
	* The singleton instance of CLTracker_WordPress.
	*
	* @access private
	* @var CLTracker_WordPress
	*/
	private static $instance;

	/**
	* The singleton instance of CLTracker.
	*
	* @access private
	* @var CLTracker
	*/
	private $cltracker;

	/**
	* The name of options array.
	*
	* @access private
	* @var string
	*/
	private $option   = 'cltracker_wordpress_options';

    /**
	* The default values for our options array.
	*
	* Not used since 1.0.0, outside of activation hooks, with our move to the Settings API.
	* See CLTracker_WordPress::register_settings().
	*
	* @access public
	* @var array
	* @since 1.0.0
	*/
	public $defaults = array(

		// Your actionrecorder app id that we'll use to initialize cltrack.js
		'app_id'           => '',

		// Whether or not we should track custom events for users logging in
		// right now we track all users logged in by default
		'track_logins'      => 1,
		'track_search_made' => 1,

		// Ecommerce events default
		'track_add_to_cart' => 1,
		'track_remove_from_cart' => 1,
		'track_product_viewed' => 1,
		'track_category_viewed' => 1,
		'track_purchased' => 1,
		'track_checkout_made' => 1

    );

    /**
	 * Returns Settings option name.
	 *
	 *
	 * @return string Settings option name
	 */
	public function get_option_name() {
		return $this->option;
	}
    
    /**
	* Retrieves CLTracker_WordPress instance
	*
	* @return object Singleton instance of CLTracker_WordPress
	*/
	public static function get_instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof CLTracker_WordPress ) ) {

			self::$instance = new CLTracker_WordPress;
			self::$instance->admin_hooks();
			self::$instance->frontend_hooks();
            self::$instance->cltracker = CLTracker::get_instance();
            self::$instance->include_files();
		}

		return self::$instance;
    }

    /**
	* Retrieves settings array.
    *
    * @return array Array of settings.
    */
	public function get_settings() {
		return apply_filters( 'cltracker_get_settings', get_option( $this->option ), $this );
	}
    
    /**
	* Updates settings array.
	*
	* @since  1.0.0
	*
	* @param  array $settings Array of settings
	* @uses   apply_filters() Applies 'cltracker_get_settings' filter to allow other developers to override.
	*
	* @deprecated Deprecated in 1.0.0
	*
	* @return array Array of settings.
	*/
	private function set_settings( $settings ) {
		return update_option( $this->option, $settings );
    }
    
    /**
	* Hooks into actions and filters that affect the administration areas.
	*
	*/
	public function admin_hooks() {
        add_action( 'admin_menu'         , array( self::$instance, 'admin_menu' ) );
        
        add_filter( 'plugin_action_links', array( self::$instance, 'plugin_action_links' ), 10, 2 );
 
		add_action( 'admin_init'         , array( self::$instance, 'register_settings' ) );
    }
    
    /**
	* Adds "Analytics" Menu item to admin area.
	*
	*/
	public function admin_menu() {

		add_options_page(
			apply_filters( 'cltracker_admin_menu_page_title', __( 'Customerlabs CDP', 'cltracker' ) ), // Page Title
			apply_filters( 'cltracker_admin_menu_menu_title', __( 'Customerlabs CDP', 'cltracker' ) ), // Menu Title
			apply_filters( 'cltracker_admin_settings_capability', 'manage_options' ),  // Capability Required
			self::SLUG,                                                              // Menu Slug
			array( $this, 'admin_page' )                                             // Function
		);

    }
    
    public function include_files() {

		do_action( 'cltracker_pre_include_files', self::$instance );

		include_once(  dirname( __FILE__ ) . '/class.cltracker-settings.php' );
		include_once(  dirname( __FILE__ ) . '/class.cltracker-cookie.php' );
		include_once(  dirname( __FILE__ ) . '/integrations/ecommerce.php' );

		do_action( 'cltracker_include_files', self::$instance );
	}

    /**
	 * The callback used to build out the admin settings area.
	 *
	 * @since 1.0.0
	 */
	public function admin_page() {

		// Make sure the user has the required permissions to view the settings.
		if ( ! current_user_can( 'manage_options' ) ) {
			wp_die( __( 'Sorry, you don\'t have the permissions to access this page.', 'cltracker' ) );
		}

		include_once( dirname( __FILE__ ) . '/templates/settings.php');
    }
    
    /**
	* Adds "Settings" link to plugin row.
	*
	* @param  array  $links Array of links on plugin action row.
	* @param  string $file  Basename of file.
	* @return array  $links Modified array of links on plugin action row.
	*/
	public function plugin_action_links( $links, $file ) {
        error_log(print_r($links, true));
		// Not for other plugins, silly. NOTE: This doesn't work properly when
		// the plugin for testing is a symlink!! If you change this, test it.
		// Note: Works fine as of 3.9, see @link: https://core.trac.wordpress.org/ticket/16953
		if ( $file != plugin_basename( __FILE__ ) ) {
			return $links;
		}

		// Add settings link to the beginning of the row of links.
		$settings_link = sprintf( '<a href="options-general.php?page=' . self::SLUG . '">%s</a>', __( 'Settings' ) );

		array_unshift( $links, $settings_link );

		return $links;
    }

    /**
	 * Registers settings for cltracker.
	 * 
	 * @return void
	 */
	public function register_settings() {

		$settings = $this->_get_default_settings();

	 	register_setting( self::SLUG, $this->get_option_name(), array( 'CLTracker_Settings', 'core_validation' ) );

		foreach ( $settings as $section_name => $section ) {

		 	add_settings_section(
				$section_name,
				$section['title'],
				$section['callback'],
				self::SLUG
			);

		 	foreach ( $section['fields'] as $field ) {

			 	add_settings_field(
					$field['name'],
					$field['title'],
					$field['callback'],
					self::SLUG,
					$section_name
				);

		 	}
		}

    }
    
    /**
	 * Returns array of settings.
	 *
	 */
	public function _get_default_settings() {
		return apply_filters( 'cltracker_default_settings', array(
				'general' => array(
					'title'    => __( 'General', 'cltracker' ),
					'callback' => array( 'CLTracker_Settings', 'general_section_callback' ),
					'fields'   => array(
						array(
							'name'            => 'app_id',
							'title'           => __( 'CustomerLabs App ID', 'cltracker' ),
							'callback'        => array( 'CLTracker_Settings', 'app_id_callback' ),
                        ),
                        
                        array(
							'name'            => 'track_logins',
							'title'           => __( 'Track logins', 'cltracker' ),
							'callback'        => array( 'CLTracker_Settings', 'track_logins_callback' ),
						),

                        array(
							'name'            => 'track_search_made',
							'title'           => __( 'Track search made', 'cltracker' ),
							'callback'        => array( 'CLTracker_Settings', 'track_search_made_callback' ),
						)
					)
				),
				'woocommerce_settings' => array(
					'title'    => __( 'WooCommerce Settings', 'cltracker' ),
					'callback' => array( 'CLTracker_Settings', 'ecommerce_section_callback' ),
					'fields'   => array(
						array(
							'name'            => 'track_add_to_cart',
							'title'           => 'Add to cart',
							'callback'        => array( 'CLTracker_Settings', 'track_add_to_cart_callback' ),
						),
						array(
							'name'            => 'track_remove_from_cart',
							'title'           => 'Remove from cart',
							'callback'        => array( 'CLTracker_Settings', 'track_remove_from_cart_callback' ),
						),
						array(
							'name'            => 'track_product_viewed',
							'title'           => 'Product viewed',
							'callback'        => array( 'CLTracker_Settings', 'track_product_viewed_callback' ),
						),
						array(
							'name'            => 'track_category_viewed',
							'title'           => 'Category viewed',
							'callback'        => array( 'CLTracker_Settings', 'track_category_viewed_callback' ),
						),
						array(
							'name'            => 'track_purchased',
							'title'           => 'Purchased',
							'callback'        => array( 'CLTracker_Settings', 'track_purchased_callback' ),
						),
						array(
							'name'            => 'track_checkout_made',
							'title'           => 'Checkout made',
							'callback'        => array( 'CLTracker_Settings', 'track_checkout_made_callback' ),
						)
					)
				)
			)
		);
    }
    
    /**
	 * hooks used to setup front-end.
	 *
	 */
	public function frontend_hooks() {

		add_action( 'wp_head'          , array( self::$instance, 'wp_head' )       , 9    );
		add_action( 'wp_footer'        , array( self::$instance, 'wp_footer' )     , 9    );
		add_action( 'login_head'       , array( self::$instance, 'wp_head' )       , 9    );
		add_action( 'login_footer'     , array( self::$instance, 'wp_footer' )     , 9    );
		add_action( 'wp_login'         , array( self::$instance, 'login_event'    ), 9, 2 );
		add_action( 'user_register'    , array( self::$instance, 'user_register'  ), 9    );
    }
    
    /**
	 * Render cltrack javascript in header.
	 *
	 */
	public function wp_head() {

		// Figure out whether the user should be ignored or not.
		$ignore = false;

		$settings = $this->get_settings();
		$user     = wp_get_current_user();

		// Render the snippet.
		self::$instance->cltracker->initialize( $settings );
    }
    
    /**
	 * Renders _cl.track() code in head.
	 */
	public function wp_footer() {

		// Identify the user if the current user merits it.
        $identify = $this->get_current_user_identify();
        
		if ( $identify ) {
			self::$instance->cltracker->identify( $identify );
		}

		// Track a custom page view event if the current page merits it.
		$tracks = $this->get_current_page_track();
		$page  = $this->get_current_page();

		if ( $tracks ) {
			$settings = $this->get_settings();
			self::$instance->cltracker->track( $tracks, $settings );
		}

    }
    
    /**
	 * Based on the current user or commenter, see if we have enough information
	 * to record an `identify` call. Since commenters don't have IDs, we
	 * identify everyone by their email address.
	 *
	 * @since  1.0.0
	 *
	 * @return bool|array Returns false if there is no commenter or logged in user
	 *                    An array of the user ID and traits if there is an authenticated user.
	 */
	public function get_current_user_identify() {
		$settings  = $this->get_settings();

		$user      = wp_get_current_user();
		$commenter = array_filter( wp_get_current_commenter() );
		$identify  = false;

		if ( is_user_logged_in() && $user && CLTracker_Cookie::get_cookie( 'logged_in' )) {
			$identify = array(
				"customProperties" => array(
					'identify_by_email' => array(
						't'		=> 'string',
						'v' 	=> $user->user_email,
						'ib'	=> true
					),
					'user_traits' => array(
						't' => 'Object',
						'v' => array(
							'username'  => $user->user_login,
							'email'     => $user->user_email,
							'first_name' => $user->user_firstname,
							'last_name'  => $user->user_lastname,
							'url'       => $user->user_url
						)
					)
				)
			);
		}
		else if ( $commenter && CLTracker_Cookie::get_cookie( 'logged_in' )) {
			$identify = array(
				"customProperties" => array(
					'identify_by_email' => array(
						't'		=> 'string',
						'v' 	=> $commenter['comment_author_email'],
						'ib'	=> true
					),
					'user_traits' => array(
						't' => 'Object',
						'v' => array(
							'email' => $commenter['comment_author_email'],
							'name'  => $commenter['comment_author'],
							'url'   => $commenter['comment_author_url']
						)
					)
				)
			);
		}

		if ( $identify ) {
			// Clean out empty traits before sending it back.
			$identify['customProperties']['user_traits'] = array_filter( $identify['customProperties']['user_traits'] );
		}

		/**
		 * Allows developers to modify the entire $identify call.
		 *
		 */
		return apply_filters( 'cltracker_get_current_user_identify', $identify, $settings, $this );
    }
    
    	/**
	 * Filters the .identify() call with the newly signed up user.
	 * This is helpful, as the user will often times not be authenticated after signing up.
	 *
	 * @since  1.0.0
	 *
	 * @param  mixed $identify   False if no user is found, array of traits and ID if a user is found.
	 * @return array $identify   Array of traits for newly signed up user.
	 */
	public function new_user_identify( $identify ) {

		if ( CLTracker_Cookie::get_cookie( 'signed_up' ) ) {

			$user_id = json_decode( CLTracker_Cookie::get_cookie( 'signed_up' ) );
			$user    = get_user_by( 'id', $user_id );

			$identify = array(
				"customProperties" => array(
					'identify_by_email' => array(
						't'		=> 'string',
						'v' 	=> $user->user_email,
						'ib'	=> true
					),
					'user_traits' => array(
						't' => 'Object',
						'v' => array(
							'username'  => $user->user_login,
							'email'     => $user->user_email,
							'first_name' => $user->user_firstname,
							'last_name'  => $user->user_lastname,
							'url'       => $user->user_url
						)
					)
				)
			);
		}

		return $identify;
    }
    
    /**
	* Used to track the current event.  Used for _cl.track().
	* @return array Array containing the page being tracked along with any additional properties.
	*/
	private function get_current_page_track() {

		$settings = $this->get_settings();

		// Login Event
		// --------
		if ( $settings['track_logins'] ) {

			$user = wp_get_current_user();
			$hash = md5( json_encode( $user ) );

			if ( CLTracker_Cookie::get_cookie( 'logged_in', $hash ) ) {
                
                add_filter( 'cltracker_get_current_user_identify', array( self::$instance, 'get_current_user_identify' ) );
                
				$track = array(
					'event'      => __( 'User Logged In', 'cltracker' ),
					'attributes' => array(),
					'cached_event' => 'logged_in'
				);

			}

		}

		if ( CLTracker_Cookie::get_cookie( 'signed_up' ) ) {

			$user_id = json_decode( CLTracker_Cookie::get_cookie( 'signed_up' ) );
			$user    = get_user_by( 'id', $user_id );

			add_filter( 'cltracker_get_current_user_identify', array( self::$instance, 'new_user_identify' ) );

			$track = array(
				'event'      => __( 'User Signed Up', 'cltracker' ),
				'attributes' => array(),
				'cached_event' => 'signed_up'
			);

		}

		// Searches
		// --------
		if ( $settings['track_search_made'] ) {
			// The search page.
			if ( is_search() ) {
				$track = array(
					'event'      => __( 'Search made', 'cltracker' ),
					'attributes' => array(
						"customProperties" => array(
							'search_made' => get_query_var( 's' )
						)
					)
				);
			}
		}

		// We don't have a page we want to track.
		if ( ! isset( $track ) ) {
			$track = false;
		}

		if ( $track ) {
			// All of these are checking for pages, and we don't want that to throw
			// off Google Analytics's bounce rate, so mark them `noninteraction`.
			// $track['attributes']['noninteraction'] = true;

			// Clean out empty attributes before sending it back.
			$track['attributes'] = array_filter( $track['attributes'] );
		}

		return apply_filters( 'cltracker_get_current_page_track', [ $track ], $settings, $this );
	}
    /**
	* Uses CLTracker_Cookie::set_cookie() to notify CLTracker that a user has logged in.
	*
	* @since  1.0.0
	*
	* @param  string  $login Username of logged in user.
	* @param  WP_User $user  User object of logged in user.
	*
	*/
	public function login_event( $login, $user ) {

		CLTracker_Cookie::set_cookie( 'logged_in', md5( json_encode( $user ) ) );
    }
    
    /**
	* Uses CLTracker_Cookie::set_cookie() to notify CLTracker that a user has signed up.
	*
	*
	* @param  string  $user_id email of new user.
	*
	*/
	public function user_register( $user_id ) {
		CLTracker_Cookie::set_cookie( 'signed_up', json_encode( $user_id ) );
    }

	/**
	* Used in our activation hook to set up our default settings.
    *
    * @since  1.0.0
    *
    * @return void
    */
    public static function setup_settings() {
		$settings = get_option( CLTracker_WordPress::get_instance()->get_option_name() );
		if ( ! empty( $settings ) ) {
				return;
		}
		update_option( CLTracker_WordPress::get_instance()->get_option_name(), CLTracker_WordPress::get_instance()->defaults );
    }
    
    public static function esc_js_deep( $value ) {
		if ( is_array( $value ) ) {
			$value = array_map( array( __CLASS__, 'esc_js_deep' ), $value );
		} elseif ( is_object( $value ) ) {
			$vars = get_object_vars( $value );
			foreach ( $vars as $key => $data ) {
				$value->{$key} = self::esc_js_deep( $data );
			}
		} elseif ( is_string( $value ) ) {
			$value = esc_js( $value );
		}

		return $value;
    }
    
    private function get_current_page() {

		$page = apply_filters( 'cltracker_get_current_page', false, $this->get_settings(), $this );

		if ( $page ) {
			$page['attributes'] = is_array( $page['attributes'] ) ? $page['attributes'] : array();
			// All of these are checking for pages, and we don't want that to throw
			// off Google Analytics's bounce rate, so mark them `noninteraction`.
			$page['attributes']['noninteraction'] = true;

			// Clean out empty attributes before sending it back.
			$page['attributes'] = array_filter( $page['attributes'] );
		}

		return $page;
	}
    
}

register_activation_hook( __FILE__, array( 'CLTracker_WordPress', 'setup_settings' ) );
add_action( 'plugins_loaded', 'CLTracker_WordPress::get_instance' );
