<?php

class CLTracker_Commerce_Woo extends CLTracker_ECommerce {

	/**
	* Init method registers two types of hooks: Standard hooks, and those fired in-between page loads.
	*
	* For all our events, we hook into either `cltracker_get_current_page` or `cltracker_get_current_page_track`
	* depending on the API we want to use.
	*
	* For events that occur between page loads, we hook into the appropriate action and set a CLTracker_Cookie
	* instance to check on the next page load.
	*
	* @access public
	* @since  1.0.0
	*
	*/
	public function init() {

		$this->register_hook( 'cltracker_get_current_page_track', 'checkout_made'    , 2, $this );
		$this->register_hook( 'cltracker_get_current_page_track', 'viewed_category'  , 2, $this );
		$this->register_hook( 'cltracker_get_current_page_track', 'viewed_product'   , 2, $this );
		$this->register_hook( 'cltracker_get_current_page_track', 'completed_order'  , 2, $this );
		$this->register_hook( 'cltracker_get_current_user_identify', 'user_did_completed_order', 2, $this );

		/* HTTP actions */
		add_action( 'woocommerce_add_to_cart'                   , array( $this, 'add_to_cart' )     , 10, 3 );
		add_action( 'woocommerce_remove_cart_item', array( $this, 'remove_from_cart' ), 10, 1);
	}

	/**
	* Adds category name to cl.track()
	*
	* @since  1.0.0
	* @access public
	*
	* @return array Filtered array of name and attributes for cl.track().
	*/
	public function viewed_category() {

		$args = func_get_args();
		$tracks = $args[0];
		$settings = $args[1];

		if ( $settings['track_category_viewed'] ) {
			if ( is_tax( 'product_cat' ) ) {
				$category = get_term_by( 'id', get_queried_object_id(), 'product_cat' );
				$track = array(
					'event'      => __( 'Category viewed', 'cltracker' ),
					'attributes' => array(
						'category_name'  => array(
							't' => 'string',
							'v' => $category->name
						)
					)
				);
				array_push($tracks, $track);
			}

		}

		return $tracks;
	}

	/**
	* Triggers product view when user visits a product page
	*
	* @since  1.0.0
	* @access public
	*
	* @return array Filtered array of name and attributes for cl.track().
	*/

	public function viewed_product() {

		$args  = func_get_args();
		$tracks = $args[0];
		$settings = $args[1];

		if ( $settings['track_product_viewed']) {
			if ( is_singular( 'product' ) ) {

					$product = wc_get_product( get_queried_object_id() );
					$image_id  = $product->get_image_id();
					$image_url = wp_get_attachment_image_url( $image_id, 'full' );
					$track = array(
						'event'      => __( 'Product viewed', 'cltracker' ),
						'attributes' => array(
							"customProperties" => array(
								"currrency" => get_woocommerce_currency(),
								"content_type" => "product_group",
								"pp" => "from pp"
							),
							"productProperties" => array(
								array(
									'product_id'       => $product->get_id(),
									'product_name'     => $product->get_title(),
									'product_price'    => $product->get_price(),
									'product_image'	   => $image_url,
									'product_sku'      => $product->get_sku(),
									'product_category' => implode( ', ', wp_list_pluck( wc_get_product_terms( $product->get_id(), 'product_cat' ), 'name' ) ),
								)
							)
						)
					);
					array_push($tracks, $track);
			}
		}

		return $tracks;
	}

	/**
	* Triggers Added to Cart event when user clicks on Add to cart button 
	* and added to cart is an ajax event, This event will store in cookie and get triggered 
	* when a page gets reloaded
	*
	* @param string $key      Key name for item in cart.
	* @param int    $id       Product id
	* @param int    $quantity item quantity
	*
	* @since  1.0.0
	* @access public
	*
	* @return array Filtered array of name and attributes for cl.track().
	*/
	public function add_to_cart( $key, $id, $quantity ) {
		
		if ( ! is_object( WC()->cart ) ) {
			return;
		}

		$items     = WC()->cart->get_cart();
		$cart_item = $items[ $key ];
		
		if ( ! is_array( $cart_item ) ) {
			return;
		}
		

		$product = wc_get_product($id);

		$image_id  = $product->get_image_id();
		$image_url = wp_get_attachment_image_url( $image_id, 'full' );

		CLTracker_Cookie::set_cookie( 'added_to_cart' . '_' . $id, json_encode(
			array(
				'event'      => __( 'Added to cart', 'cltracker' ),
				'attributes' => array(
					"customProperties" => array(
						"currency" => get_woocommerce_currency(),
						"content_type" => "product_group"
					),
					"productProperties" => array(
						array(
							'product_id'       => $id,
							'product_quantity' => $quantity,
							'product_name'     => $product->get_name(),
							'product_price'    => $product->get_price(),
							'product_image'	   => $image_url,
							'product_sku'      => $product->get_sku(),
							'product_category' => implode( ', ', wp_list_pluck( wc_get_product_terms( $product->get_id(), 'product_cat' ), 'name' ) ),
						)
					)
				),
				'cached_event' => 'added_to_cart_' . $id
			)
		));

	}
	
	/**
	* Triggers Removed to Cart event when user clicks on remove button from the cart
	* and removed from cart is an ajax event, This event will store in cookie and get triggered 
	* when a page gets reloaded
	*
	* @param string $key      Key name for item in cart.
	*
	* @since  1.0.0
	* @access public
	*
	* @return array Filtered array of name and attributes for cl.track().
	*/
	public function remove_from_cart($key) {

		if ( ! is_object( WC()->cart ) ) {
			return;
		}
		
		$items     = WC()->cart->get_cart();
		$cart_item = $items[ $key ];
		
		if ( ! is_array( $cart_item ) ) {
			return;
		}
		
		$product = wc_get_product($cart_item['product_id']);

		CLTracker_Cookie::set_cookie( 'removed_from_cart' . '_' . $cart_item['product_id'], json_encode(
			array(
				'event'      => __( 'Removed from cart', 'cltracker' ),
				'attributes' => array(
					"customProperties" => array(
						"currency" => get_woocommerce_currency(),
						"content_type" => "product_group"
					),
					"productProperties" => array(
						array(
							'product_id'       => $cart_item['product_id'],
							'product_quantity' => 0,
							'product_name'     => $product->get_name(),
							'product_price'    => $product->get_price(),
							'product_category' => implode( ', ', wp_list_pluck( wc_get_product_terms( $product->get_id(), 'product_cat' ), 'name' ) ),
						)
					)
				),
				'cached_event' => 'removed_from_cart_' . $id
			)
		));
	}

	/**
	* Triggers after a successful purchase, tracks list of products on purchased page
	*
	* @since  1.0.0
	* @access public
	*
	* @return array Filtered array of name and attributes for cl.track().
	*/
	public function completed_order() {
		$args  = func_get_args();
		$tracks = $args[0];
		$settings = $args[1];

		if ( $settings['track_purchased'] ) {

			if ( did_action( 'woocommerce_thankyou' ) ) {

				$order_number = get_query_var( 'order-received' );

				$order = new WC_Order( $order_number );

				/* Because gateways vary wildly in their usage of the status concept, we check for failure rather than success. */
				if ( 'failed' !== $order->get_status() ) {

					$items        = $order->get_items();
					$products     = array();

					foreach ( $items as $item ) {
						$_product = $item->get_product();
						$product = array(
							'product_id'       => $item->get_product_id(),
							'product_sku'      => $_product->get_sku(),
							'product_name'     => $item['name'],
							'product_price'    => $item['line_subtotal'],
							'product_quantity' => $item['qty'],
							'product_category' => implode( ', ', wp_list_pluck( wc_get_product_terms( $item->get_product_id(), 'product_cat' ), 'name' ) ),
						);

						$products[] = $product;

					}

					$track = array(
						'event'      => __( 'Purchased', 'cltracker' ),
						'attributes' => array(
							'customProperties'	=> array(
								'transaction_id'       	=> $order->get_order_number(),
								'total'    				=> $order->get_total(),
								'revenue'  				=> $order->get_total() - ( $order->get_total_shipping() + $order->get_total_tax() ),
								'value'  				=> $order->get_total() - ( $order->get_total_shipping() + $order->get_total_tax() ),
								'shipping' 				=> $order->get_total_shipping(),
								'tax'      				=> $order->get_total_tax(),
								'currency'              => $order->get_currency(),
								'discount'				=> $order->get_discount_total(),
								'content_type'          => 'product_group'
							),
							'productProperties' => $products
						)
					);
					array_push($tracks, $track);
				}
			}
		}

		return $tracks;
	}

	/**
	* Triggers after a successful purchase, identify user who did purchase
	*
	* @since  1.1.0
	* @access public
	*
	* @return array Filtered array of user_id and traits for cl.identify().
	*/
	public function user_did_completed_order() {
		$args  = func_get_args();
		$identify = $args[0];
		$settings = $args[1];

		if ( $settings["track_logins"] ) {

			if ( did_action( 'woocommerce_thankyou' ) ) {

				$order_number = get_query_var( 'order-received' );

				$order = new WC_Order( $order_number );

				/* Because gateways vary wildly in their usage of the status concept, we check for failure rather than success. */
				if ( 'failed' !== $order->get_status() ) {
					$user_traits = array();
					$user_traits["email"] = array(
						't' => 'string',
						'v' => $order->get_billing_email()
					);
					if($order->get_billing_phone()){
						$user_traits["phone"] = array(
							't' => 'string',
							'v' => $order->get_billing_phone()
						);
					}
					if($order->get_billing_first_name()){
						$user_traits['first_name'] = array(
							't' => 'string',
							'v' => $order->get_billing_first_name()
						);
					}
					if($order->get_billing_last_name()){
						$user_traits['last_name']  = array(
							't' => 'string',
							'v' => $order->get_billing_last_name()
						);
					}
					if($order->get_billing_address_1()){
						$user_traits['address_1']  = array(
							't' => 'string',
							'v' => $order->get_billing_address_1()
						);
					}
					if($order->get_billing_address_2()){
						$user_traits['address_2']  = array(
							't' => 'string',
							'v' => $order->get_billing_address_2()
						);
					}
					if($order->get_billing_city()){
						$user_traits['city']  = array(
							't' => 'string',
							'v' => $order->get_billing_city()
						);
					}
					if($order->get_billing_state()){
						$user_traits['state']  = array(
							't' => 'string',
							'v' => $order->get_billing_state()
						);
					}
					if($order->get_billing_country()){
						$user_traits['country']  = array(
							't' => 'string',
							'v' => $order->get_billing_country()
						);
					}
					if($order->get_billing_postcode()){
						$user_traits['postal_code']  = array(
							't' => 'string',
							'v' => $order->get_billing_postcode()
						);
					}

					$identify = array(
						"customProperties" => array(
							'identify_by_email' => array(
								't'		=> 'string',
								'v' 	=> $order->get_billing_email(),
								'ib'	=> true
							),
							'user_traits' => array(
								't' => 'Object',
								'v' => $user_traits
							)
						)
					);
				}
			}

		}

		return $identify;
	}


	/**
	 * Triggers after landed on checkout page
	 *
	 * @since 1.1.0
	 * @access public
	 *
	 * @return array Filtered array of name and attributes for cl.track().
	 */
	public function checkout_made(){

		$args  = func_get_args();
		$tracks = $args[0];
		$settings = $args[1];

		if ( $settings['track_checkout_made'] ) {

			if ( ! is_object( WC()->cart ) ) {
				return $tracks;
			}

			if ( !is_checkout() ) {
				return $tracks;
			}

			$cart  = WC()->cart;

			$items = $cart->get_cart();

			if ( count($items) == 0 ){
				return $tracks;
			}

			$products     = array();

			foreach ( $items as $item ) {
				$_product = wc_get_product($item['product_id']);

				$product = array(
					'product_id'       => $item['product_id'],
					'product_sku'      => $_product->get_sku(),
					'product_name'     => $_product->get_name(),
					'product_price'    => $item['line_subtotal'],
					'product_quantity' => $item['quantity'],
					'product_category' => implode( ', ', wp_list_pluck( wc_get_product_terms( $item['product_id'], 'product_cat' ), 'name' ) ),
				);

				$products[] = $product;

			}

			$track = array(
				'event'      => __( 'Checkout made', 'cltracker' ),
				'attributes' => array(
					'customProperties'	=> array(
						'total'    				=> $cart->get_total("number"),
						'shipping' 				=> $cart->get_shipping_total(),
						'tax'      				=> $cart->get_total_tax(),
						'currency'              => get_woocommerce_currency(),
						'discount'				=> $cart->get_discount_total(),
						'content_type'          => 'product_group'
					),
					'productProperties'	=> $products
				)
			);
			array_push($tracks, $track);
		}

		return $tracks;
	}

}

function cltracker_commerce_woo() {
	$commerce = new CLTracker_Commerce_Woo();

	return $commerce->init();
}

add_action( 'plugins_loaded', 'cltracker_commerce_woo', 100 );